/* This file is part of the KDE project
   Copyright (C) 2002-2003 Nadeem Hasan <nhasan@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or ( at your option ) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#include <qlayout.h>
#include <qgroupbox.h>
#include <qhbox.h>
#include <qlabel.h>
#include <qtoolbutton.h>
#include <qtooltip.h>

#include <kglobal.h>
#include <klocale.h>
#include <kiconloader.h>

#include "browserdlg.h"

BrowserDlg::BrowserDlg( CallLog *callLog, QWidget *parent, 
    const char *name )
    : KDialogBase( parent, name, true, i18n( "Call Browser" ),
      Close, Close, true ),
      m_index( 1 ), m_callLog( callLog )
{
  QWidget *w = makeMainWidget();

  QVBoxLayout *vl = new QVBoxLayout( w, 0, spacingHint() );

  QFont font = w->font();
  font.setBold( true );

  QHBox *hbox = new QHBox( w );

  vl->addWidget( hbox );

  m_lblCount = new QLabel( hbox  );
  m_lblCount->setFont( font );
  m_lblNew = new QLabel( hbox  );
  m_lblNew->setFont( font );
  m_lblNew->setAlignment( Qt::AlignRight );

  QGroupBox *grp = new QGroupBox( w );
  grp->setColumnLayout( 1, Qt::Horizontal );
  grp->setFrameShape( QFrame::WinPanel );
  grp->setFrameShadow( QFrame::Sunken );

  vl->addWidget( grp, 1 );

  m_lblName = new QLabel( grp );
  QFont nameFont = m_lblName->font();
  nameFont.setBold( true );
  nameFont.setPointSize( nameFont.pointSize()+2 );
  m_lblName->setFont( nameFont );

  m_lblNmbr = new QLabel( grp );
  m_lblNmbr->setFont( font );

  m_lblTime = new QLabel( grp );
  font = m_lblTime->font();
  font.setBold( true );
  m_lblTime->setFont( font );

  QHBoxLayout *hl = new QHBoxLayout( vl, 2 );

  m_pbFirst = new QToolButton( w );
  m_pbFirst->setIconSet( SmallIconSet( "start" ) );
  QToolTip::add( m_pbFirst, i18n( "First call" ) );
  connect( m_pbFirst, SIGNAL( clicked() ), SLOT( slotGoFirst() ) );
  hl->addWidget( m_pbFirst );

  m_pbPrev = new QToolButton( w );
  m_pbPrev->setIconSet( SmallIconSet( "back" ) );
  QToolTip::add( m_pbPrev, i18n( "Previous call" ) );
  connect( m_pbPrev, SIGNAL( clicked() ), SLOT( slotGoPrev() ) );
  hl->addWidget( m_pbPrev );

  m_pbNext = new QToolButton( w );
  m_pbNext->setIconSet( SmallIconSet( "forward" ) );
  QToolTip::add( m_pbNext, i18n( "Next call" ) );
  connect( m_pbNext, SIGNAL( clicked() ), SLOT( slotGoNext() ) );
  hl->addWidget( m_pbNext );

  m_pbLast = new QToolButton( w );
  m_pbLast->setIconSet( SmallIconSet( "finish" ) );
  QToolTip::add( m_pbLast, i18n( "Last call" ) );
  connect( m_pbLast, SIGNAL( clicked() ), SLOT( slotGoLast() ) );
  hl->addWidget( m_pbLast );

  hl->addSpacing( 120 );

  m_pbDel = new QToolButton( w );
  m_pbDel->setIconSet( SmallIconSet( "button_cancel" ) );
  QToolTip::add( m_pbDel, i18n( "Delete call" ) );
  connect( m_pbDel, SIGNAL( clicked() ), SLOT( slotDelCall() ) );
  hl->addWidget( m_pbDel );

  setFixedSize( sizeHint() );

  m_callLog->first();
  showCall();
}

BrowserDlg::~BrowserDlg()
{
}

void BrowserDlg::enableButtons()
{
  if ( m_callLog->atFirst() )
  {
    m_pbFirst->setEnabled( false );
    m_pbPrev->setEnabled( false );
  }
  else
  {
    m_pbFirst->setEnabled( true );
    m_pbPrev->setEnabled( true );
  }

  if ( m_callLog->atLast() )
  {
    m_pbNext->setEnabled( false );
    m_pbLast->setEnabled( false );
  }
  else
  {
    m_pbNext->setEnabled( true );
    m_pbLast->setEnabled( true );
  }
}

void BrowserDlg::showCall()
{
  CallData *data = m_callLog->current();

  m_lblCount->setText( QString( "%1/%2" )
      .arg( m_index )
      .arg( m_callLog->count() ) );

  if ( data->isNew() )
    m_lblNew->setText( i18n( "New Call" ) );
  else
    m_lblNew->setText( QString::null );

  m_lblName->setText( data->name() );
  m_lblNmbr->setText( data->nmbr() );
  m_lblTime->setText( KGlobal::locale()->formatDateTime( data->time() ) );

  enableButtons();

  data->setNew( false );
}

void BrowserDlg::slotGoFirst()
{
  if ( !m_callLog->atFirst() )
  {
    m_callLog->first();
    m_index = 1;
    showCall();
  }
}

void BrowserDlg::slotGoPrev()
{
  if ( !m_callLog->atFirst() )
  {
    m_callLog->prev();
    --m_index;
    showCall();
  }
}

void BrowserDlg::slotGoNext()
{
  if ( !m_callLog->atLast() )
  {
    m_callLog->next();
    ++m_index;
    showCall();
  }
}

void BrowserDlg::slotGoLast()
{
  if ( !m_callLog->atLast() )
  {
    m_callLog->last();
    m_index = m_callLog->count();
    showCall();
  }
}

void BrowserDlg::slotDelCall()
{
  m_callLog->remove();
  showCall();
}

#include "browserdlg.moc"

/* vim: et sw=2 ts=2
*/

