/* This file is part of the KDE project
   Copyright (C) 2002-2003 Nadeem Hasan <nhasan@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or ( at your option ) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#include <unistd.h>

#include <kaudioplayer.h>
#include <kstandarddirs.h>
#include <kmessagebox.h>
#include <klocale.h>

#include <kdebug.h>

#include "modem.h"
#include "kallersconfig.h"
#include "callerid.h"

CallerId::CallerId( QObject *parent, const char *name )
    : QObject( parent, name ),
      m_offline( true ), m_modem( 0 ), m_curCallData( 0 ), m_callLog( 0 )
{
  m_config = KallersConfig::instance();

  m_ringFile = locate( "appdata", "sound/ring.wav" );

  m_callLog = new CallLog();

  initModem();
  initRegExp(); 

  connect( m_config, SIGNAL( modemDeviceChanged() ), SLOT( initModem() ) );
}

CallerId::~CallerId()
{
  delete m_callLog;
  delete m_modem;
}

void CallerId::initModem()
{
  if ( m_modem )
    delete m_modem;

  m_modem = new Modem( this, "m_modem" );
  m_modem->setDevice( m_config->device() );
  m_modem->setSpeed( 38400 );

  openModem();
}

void CallerId::openModem()
{
  m_modem->open();

  if ( m_modem->isOpen() )
  {
    m_modem->writeLine( "ATZ" );
    sleep( 2 );
    m_modem->writeLine( m_config->command().latin1() );

    setOffline( false );

    connect(  m_modem, SIGNAL( gotLine(  const char * ) ),
        SLOT( slotGotLine( const char * ) ) );
  }
  else
  {
    KMessageBox::detailedError( 0, i18n( "Could not access modem device." ),
      i18n( "There was an error accessing your modem. Please check if\n"
            "your modem is functioning and you have selected the correct\n"
            "device name in the configuration. If you are using an external\n"
            "modem, also make sure that it is turned on." ) );
    setOffline( true );
  }
}

void CallerId::closeModem()
{
  if ( m_modem && m_modem->isOpen() )
  {
    disconnect( m_modem, 0, 0, 0 );
    m_modem->close();
    setOffline( true );
  }
}

void CallerId::slotOffline()
{
  if ( offline() )
    openModem();
  else
    closeModem();
}

void CallerId::initRegExp()
{
  m_reName.setPattern( "^NAME\\s*=\\s*(.*)\\s*[\r\n]$" );
  m_reNmbr.setPattern( "^NMBR\\s*=\\s*([0-9]{3})([0-9]{3})([0-9]{4})[\r\n]$" );
  m_reDate.setPattern( "^DATE\\s*=\\s*([0-9]{2})([0-9]{2})[\r\n]$" );
  m_reTime.setPattern( "^TIME\\s*=\\s*([0-9]{2})([0-9]{2})[\r\n]$" );
  m_reMesg.setPattern( "^MESG\\s*=\\s*(\\S+)\\s*[\r\n]$" );
}

void CallerId::addLogEntry( CallData *data )
{
  if ( m_config->logAnonymous() || !data->isAnonymous() )
  {
    while ( m_callLog->count() >= m_config->entryLimit() )
      m_callLog->removeLast();

    checkAnonymous();
    data->setNew( true );
    m_callLog->add( data );
  }
}

void CallerId::slotGotLine( const char *data )
{
  QString line = QString( data ).stripWhiteSpace();

  kdDebug( 12005 ) << "CallerId::slotGotLine(): " << line << endl;

  if ( line == "RING" )
  {
    if ( m_curCallData )
    {
      addLogEntry( m_curCallData );
      emit callReceived( m_curCallData );

      m_curCallData = 0;
    }

    if ( m_config->ringOnCall() )
      KAudioPlayer::play( m_ringFile );
  }
  else if ( m_reName.search( line ) != -1 )
  {
    if ( m_curCallData == 0 )
      m_curCallData = new CallData();

    m_curCallData->setName( m_reName.cap( 1 ).stripWhiteSpace() );
  }
  else if ( m_reNmbr.search( line ) != -1 )
  {
    if ( m_curCallData == 0 )
      m_curCallData = new CallData();

    m_curCallData->setNmbr( QString( "%1-%2-%3" )
              .arg( m_reNmbr.cap( 1 ) )
              .arg( m_reNmbr.cap( 2 ) ) 
              .arg( m_reNmbr.cap( 3 ) ) );
  }
  else if ( m_reMesg.search( line ) != -1 )
  {
    if ( m_curCallData == 0 )
      m_curCallData = new CallData();

    m_curCallData->setMesg( m_reMesg.cap( 1 ).stripWhiteSpace() );
  }
  else if ( m_reDate.search( line ) != -1 )
  {
    if ( m_curCallData == 0 )
      m_curCallData = new CallData();

    m_curCallData->setDate( m_reDate.cap( 1 ).toInt(), 
              m_reDate.cap( 2 ).toInt() );
  }
  else if ( m_reTime.search( line ) != -1 )
  {
    if ( m_curCallData == 0 )
      m_curCallData = new CallData();

    m_curCallData->setTime( m_reTime.cap( 1 ).toInt(),
              m_reTime.cap( 2 ).toInt() );
  }
}

void CallerId::setOffline( bool b )
{
  m_offline = b;
  emit offlineChanged( b );
}

void CallerId::clearCallLog()
{
  m_callLog->clear();
}

void CallerId::checkAnonymous()
{
  QString name = m_curCallData->name();

  if ( name.isEmpty() )
  {
    m_curCallData->setName( i18n( "Unknown Name" ) );
  }
  else if ( name == "P" )
  {
    m_curCallData->setName( i18n( "Private Name" ) );
    m_curCallData->setNmbr( i18n( "Private Number" ) );

    m_curCallData->setAnonymous( true );
  }
  else if ( name == "O" )
  {
    m_curCallData->setName( i18n( "Out of Area" ) );
  }
  else if ( name == "A" )
  {
    m_curCallData->setName( i18n( "Unknown Name" ) );
    m_curCallData->setNmbr( i18n( "Unknown Number" ) );

    m_curCallData->setAnonymous( true );
  }
}

#include "callerid.moc"

/* vim: et sw=2 ts=2
*/
