/* This file is part of the KDE project
   Copyright (C) 2002-2003 Nadeem Hasan <nhasan@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or ( at your option ) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#include <qfile.h>
#include <qdom.h>

#include <kstandarddirs.h>
#include <ksavefile.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <kdebug.h>

#include "callerid.h"
#include "calllog.h"


CallData::CallData()
    : m_anonCall( false ), m_newCall( false )
{
}

void CallData::setTime( int hr, int min )
{
  QTime time = QTime(  hr, min );
  m_callTime.setTime(  time );
}

void CallData::setDate( int mon, int day )
{
  int yr = QDate::currentDate().year();
  QDate date = QDate( yr, mon, day );
  m_callTime.setDate( date );
}

void CallData::setDateTime( const QString &s )
{
  m_callTime = QDateTime::fromString( s );
}

CallLog::CallLog()
{
  m_fileName = locateLocal( "data", "kallers/calllog.xml" );
  setAutoDelete( true );

  parse();
}

CallLog::~CallLog()
{
  save();
}

void CallLog::add( CallData *data )
{
  QPtrList<CallData>::prepend( data );
  save();
}

void CallLog::parse()
{
  QDomDocument doc;
  QDomElement rootElem;

  QFile f( m_fileName );
  if ( f.open( IO_ReadOnly ) )
  {
    doc.setContent( &f );
    rootElem = doc.documentElement();
    f.close();
  }

  if ( rootElem.isNull() )
  {
    doc = QDomDocument( "kallers" );
    rootElem = doc.createElement( "CallLog" );
    doc.appendChild( rootElem );
  }

  QDomNode node = rootElem.firstChild();
  while ( !node.isNull() )
  {
    QDomElement elem = node.toElement();
    CallData *data = new CallData();
    data->setName( elem.namedItem( "Name" ).toElement().text() );
    data->setNmbr( elem.namedItem( "PhoneNumber" ).toElement().text() );
    data->setDateTime( elem.namedItem( "DateTime" ).toElement().text() );

    append( data );

    node = node.nextSibling();
  }
}

QDomDocument CallLog::createDom()
{
  QDomDocument doc( "kallers" );
  QDomElement rootElem = doc.createElement( "CallLog");
  doc.appendChild( rootElem );

  CallData *data = first();

  while ( data )
  {
    QDomElement callDataElem = doc.createElement( "CallData" );

    QDomElement elem;

    elem = doc.createElement( "Name" );
    elem.appendChild( doc.createTextNode( data->name() ) );
    callDataElem.appendChild( elem );

    elem = doc.createElement( "PhoneNumber" );
    elem.appendChild( doc.createTextNode( data->nmbr() ) );
    callDataElem.appendChild( elem );

    elem = doc.createElement( "DateTime" );
    elem.appendChild( doc.createTextNode( data->time().toString() ) );
    callDataElem.appendChild( elem );

    rootElem.appendChild( callDataElem );

    data = next();
  }

  return doc;
}

void CallLog::save()
{
  QDomDocument doc = createDom();

  QString data = doc.toString();

  if ( !data.isEmpty() )
  {
    KSaveFile f( m_fileName );

    if ( f.status() != 0 )
    {
      KMessageBox::error( 0L, i18n( "Could not save call log in %1." )
            .arg( m_fileName ) );
      return;
    }

    f.file()->writeBlock( data, data.length() );

    if ( !f.close() )
      KMessageBox::error( 0L, i18n( "Could not save call log in %1." )
            .arg( m_fileName ) );
  }
}

/* vim: et sw=2 ts=2
*/
