/* This file is part of the KDE project
   Copyright (C) 2002-2003 Nadeem Hasan <nhasan@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or ( at your option ) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#include <qvbox.h>
#include <qlabel.h>
#include <qtooltip.h>

#include <kapplication.h>
#include <kiconloader.h>
#include <kpassivepopup.h>
#include <klocale.h>
#include <kpopupmenu.h>
#include <kaction.h>
#include <khelpmenu.h>
#include <kmessagebox.h>
#include <kdebug.h>

#include "dockwidget.h"
#include "callerid.h"
#include "kallersconfig.h"
#include "kallersconfdialog.h"
#include "browserdlg.h"

DockWidget::DockWidget( QWidget *parent, const char *name )
    : KSystemTray( parent, name ),
      m_callerId( 0 ), m_config( 0 ), m_actClearLog( 0 ), 
      m_actRingOnCall( 0 )
{
  m_config = KallersConfig::instance();

  m_dockPixmap = UserIcon( "kallers_dock" );
  m_miniPixmap = SmallIcon( "kallers" );

  setPixmap( m_dockPixmap );

  m_callerId = new CallerId( this, "m_callerId" );

  initContextMenu();

  connect( m_callerId, SIGNAL( callReceived( const CallData * ) ),
          SLOT( slotCallReceived( const CallData * ) ) );
  connect( m_callerId, SIGNAL( offlineChanged( bool ) ),
          SLOT( slotOfflineChanged( bool ) ) );

  updateToolTip();
}

DockWidget::~DockWidget()
{
}

void DockWidget::initContextMenu()
{
  KPopupMenu *popup = contextMenu();
  KActionCollection *ac = new KActionCollection( this );

  m_actClearLog = new KAction( i18n( "C&lear Call Log" ) );
  connect( m_actClearLog, SIGNAL( activated() ), SLOT( slotClearLog() ) );

  m_actClearLog->plug( popup );

  m_actRingOnCall = new KToggleAction( i18n( "&Ring on Call" ) );
  connect( m_actRingOnCall, SIGNAL( toggled( bool ) ), 
          SLOT( slotToggleRing() ) );
  m_actRingOnCall->setChecked( m_config->ringOnCall() );
  m_actRingOnCall->plug( popup );

  popup->insertSeparator();

  m_actOffline = new KToggleAction( i18n( "&Offline" ) );
  connect( m_actOffline, SIGNAL( toggled( bool ) ),
          m_callerId, SLOT( slotOffline() ) );
  m_actOffline->setChecked( m_callerId->offline() );
  m_actOffline->plug( popup );

  popup->insertSeparator();

  m_actViewCalls = new KAction( i18n( "&Browse Call Log..." ), 
          QString( "contents" ) );
  connect( m_actViewCalls, SIGNAL( activated() ), SLOT( slotViewCalls() ) );
  m_actViewCalls->plug( popup );

  popup->insertSeparator();

  KAction *actPrefs = KStdAction::preferences( this, SLOT( slotPrefs() ),
        ac );
  actPrefs->plug( popup );

  KHelpMenu *help = new KHelpMenu( this, KGlobal::instance()->aboutData(),
          false );
  popup->insertItem( SmallIconSet( "help" ), i18n( "&Help" ), help->menu() );

  updateActions();
}

void DockWidget::slotCallReceived( const CallData *callData )
{
  kdDebug( 12005 ) << "Received call from: " << callData->name() << endl;

  updateToolTip();
  updateActions();

  KPassivePopup *popup = new KPassivePopup( winId(), "popup" );
  popup->setAutoDelete( true );
  popup->setTimeout( 15 * 1000 );

  QVBox *vbox = new QVBox( popup );
  QHBox *hbox = new QHBox( vbox );
  hbox->setSpacing( 6 );
  QLabel *icon = new QLabel( hbox );
  icon->setPixmap( m_miniPixmap );
  QLabel *name = new QLabel( callData->name(), hbox );
  QFont font = name->font();
  font.setBold( true );
  name->setFont( font );
  hbox->setStretchFactor( name, 1 );
  ( void ) new QLabel( callData->nmbr(), vbox );

  popup->setView( vbox );

  connect( popup, SIGNAL( clicked() ), popup, SLOT( hide() ) );

  popup->show();
}

void DockWidget::slotClearLog()
{
  int canClear = KMessageBox::warningYesNo( 0L, 
          i18n( "Permanently clear call log?" ) );

  if ( canClear == KMessageBox::Yes )
    m_callerId->clearCallLog();

  updateToolTip();
  updateActions();
}

void DockWidget::slotToggleRing()
{
  m_config->setRingOnCall( !m_config->ringOnCall() );

  m_config->save();
}

void DockWidget::slotOfflineChanged( bool b )
{
  m_actOffline->setChecked( b );
  updateToolTip();
}

void DockWidget::slotPrefs()
{
  KallersConfDialog *confDlg;
  confDlg = new KallersConfDialog( this, "confDlg" );

  connect( confDlg, SIGNAL( configChanged() ), SLOT( slotConfigChanged() ) );

  confDlg->exec();

  delete confDlg;
}

void DockWidget::slotConfigChanged()
{
  m_actRingOnCall->setChecked( m_config->ringOnCall() );
}

void DockWidget::slotViewCalls()
{
  BrowserDlg *dlg = new BrowserDlg( m_callerId->callLog(), this );

  dlg->exec();
  updateToolTip();

  delete dlg;
}

void DockWidget::updateActions()
{
  uint numCalls = m_callerId->numCalls();
  m_actClearLog->setEnabled( numCalls > 0 );
  m_actViewCalls->setEnabled( numCalls > 0 );
}

void DockWidget::updateToolTip()
{
  QString toolTip;
  uint newCalls = m_callerId->newCalls();
  uint numCalls = m_callerId->numCalls();

  if ( newCalls == 0 )
    toolTip = i18n( "No new calls, %1 total" )
        .arg( numCalls );
  else if ( newCalls == 1 )
    toolTip = i18n( "1 new call, %1 total" )
        .arg( numCalls );
  else
    toolTip = i18n( "%1 new calls, %2 total" )
        .arg( newCalls )
        .arg( numCalls );

  if ( m_callerId->offline() )
    toolTip += i18n( " (offline)" );

  QToolTip::remove( this );
  QToolTip::add( this, toolTip );
}

#include "dockwidget.moc"

/* vim: et sw=2 ts=2
*/

