/*
    KMLOCfg

    A utility to configure the ELSA MicroLink(tm) Office modem.

    Copyright (C) 2000 Oliver Gantz <Oliver.Gantz@epost.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    ------
    ELSA and MicroLink are trademarks of ELSA AG, Aachen.
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <termios.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <pwd.h>
#include <errno.h>

#include <qglobal.h>

#include "modem.h"


#define LOCK_PATH "/var/lock"


#ifndef CSOH
#define CSOH  01
#endif

#ifndef CSTX
#define CSTX  02
#endif

#ifndef CEOT
#define CEOT  04
#endif

#ifndef CACK
#define CACK  06
#endif

#ifndef CNAK
#define CNAK 025
#endif

#ifndef CCAN
#define CCAN 030
#endif



Modem::Modem(QObject *parent, const char *name) : QObject(parent, name)
{
  mOpen = false;

	init();
}


Modem::~Modem()
{
	close();
}


void Modem::setDevice(const QString& name)
{
	if (fdev)
		free(fdev);

	fdev = strdup(name.latin1());
}


void Modem::setSpeed(int speed)
{
	switch (speed) {
		case 300:
			cspeed = B300;
			break;
		case 600:
			cspeed = B600;
			break;
		case 1200:
			cspeed = B1200;
			break;
		case 2400:
			cspeed = B2400;
			break;
		case 4800:
			cspeed = B4800;
			break;
		case 9600:
			cspeed = B9600;
			break;
		case 19200:
			cspeed = B19200;
			break;
		case 38400:
			cspeed = B38400;
			break;
#ifdef B57600
		case 57600:
			cspeed = B57600;
			break;
#endif
#ifdef B115200
		case 115200:
			cspeed = B115200;
			break;
#endif
#ifdef B230400
		case 230400:
			cspeed = B230400;
			break;
#endif
		default:
			cspeed = B38400;
	}
}


void Modem::setData(int data)
{
	cflag &= ~CSIZE;

	switch (data) {
		case 5:
			cflag |= CS5;
			break;
		case 6:
			cflag |= CS6;
			break;
		case 7:
			cflag |= CS7;
			break;
		default:
			cflag |= CS8;
	}
}


void Modem::setParity(char parity)
{
	cflag &= ~(PARENB | PARODD);

	if (parity == 'E')
		cflag |= PARENB;
	else if (parity == 'O')
		cflag |= PARENB | PARODD;
}


void Modem::setStop(int stop)
{
	if (stop == 2)
		cflag |= CSTOPB;
	else
		cflag &= ~CSTOPB;
}


bool Modem::open()
{
	struct termios tty;

	close();

	if (!lockDevice()) {
		return false;
	}

	if ((fd = ::open(fdev, O_RDWR | O_NOCTTY | O_NONBLOCK)) == -1) {
		return false;
	}
	tcflush(fd, TCIOFLUSH);
	if (tcgetattr(fd, &init_tty) == -1) {
		::close(fd);
		fd = 0;
		return false;
	}
	memset(&tty, 0, sizeof(tty));
	tty.c_iflag = IGNBRK | IGNPAR;
	tty.c_oflag = 0;
	tty.c_cflag = cflag;
	tty.c_lflag = 0;
	cfsetospeed(&tty, cspeed);
	cfsetispeed(&tty, cspeed);
	tcdrain(fd);
	if (tcsetattr(fd, TCSANOW, &tty) == -1) {
		::close(fd);
		fd = 0;
		return false;
	}

	sn = new QSocketNotifier(fd, QSocketNotifier::Read, this, "modemsocketnotifier");
	CHECK_PTR(sn);
	connect(sn, SIGNAL(activated(int)), SLOT(readChar(int)));

        mOpen = true;

	return true;
}


void Modem::close()
{
	delete sn;
	sn = 0;

	if (fd) {
		tcflush(fd, TCIOFLUSH);
		tcsetattr(fd, TCSANOW, &init_tty);
		::close(fd);
		fd = 0;
	}

	unlockDevice();

  mOpen = false;
}


void Modem::flush()
{
	if (fd) {
		tcflush(fd, TCIOFLUSH);
		bufpos = 0;
	}
}


bool Modem::lockDevice()
{
	char *p;
	char fname[1024];
	char content[256];
	ssize_t count;
	pid_t pid;
	int lfd;
	struct passwd *pw;

	if (is_locked)
		return true;

	if ((p = strrchr(fdev, '/')))
		p++;
	else
		p = fdev;

	sprintf(fname, "%s/LCK..%s", LOCK_PATH, p);
	if (!access(fname, F_OK)) {
		if ((lfd = ::open(fname, O_RDONLY)) < 0) {
			return false;
		}

		count = read(lfd, content, 79);
		if (count < 0) {
			::close(lfd);
			return false;
		}
		content[count] = 0;
		::close(lfd);

		count = sscanf(content, "%d", &pid);
		if ((count != 1) || (pid <= 0)) {
			return false;
		}

		if (!kill((pid_t)pid, 0)) {
			return false;
		}

		if (errno != ESRCH) {
			return false;
		}
	}
	
	if ((lfd = creat(fname, 0644)) == -1) {
		return false;
	}

	pid = (int)getpid();
	pw = getpwuid(getuid());
	sprintf(content, "%08d %s %s", pid, "kmlofax", pw->pw_name);
	write(lfd, content, strlen(content));
	::close(lfd);

	is_locked = true;

	return true;
}


void Modem::unlockDevice()
{
	char *p;
	char fname[1024];

	if (is_locked) {
		if ((p = strrchr(fdev, '/')))
			p++;
		else
			p = fdev;

		sprintf(fname, "%s/LCK..%s", LOCK_PATH, p);
		unlink(fname);
		is_locked = false;
	}
}


bool Modem::dsrOn()
{
	int flags;

	if (!fd) {
		return false;
	}

	if (ioctl(fd, TIOCMGET, &flags) == -1) {
		return false;
	}

	return (flags & TIOCM_DSR) != 0;
}


bool Modem::ctsOn()
{
	int flags;

	if (!fd) {
		return false;
	}

	if (ioctl(fd, TIOCMGET, &flags) == -1) {
		return false;
	}

	return (flags & TIOCM_CTS) != 0;
}


void Modem::writeChar(const char c)
{
	write(fd, (const void *)&c, 1);
}


void Modem::writeLine(const char *line)
{
	write(fd, (const void *)line, strlen(line));
	writeChar('\r');
}


void Modem::readChar(int)
{
	uchar c;

	while (read(fd, (void *)&c, 1) == 1) {
		if (c == '\n') {
			buffer[bufpos] = 0;
			bufpos = 0;
			emit gotLine(( const char * ) buffer );
			break;
		} else if ((bufpos < 1000) && (c != '\r'))
			buffer[bufpos++] = c;
	}
}


void Modem::init()
{
	is_locked = false;

	fdev = 0;
	fd = 0;
	sn = 0;

	cspeed = B38400;
// No flow control
	cflag = CS8 | CREAD | CLOCAL;
//	cflag = CS8 | CREAD | CLOCAL | CRTSCTS;

	bufpos = 0;
}


#include "modem.moc"
